"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.batchWithSize = exports.toGroupUows = exports.group = exports.unBatchUow = exports.toBatchUow = void 0;

var _highland = _interopRequireDefault(require("highland"));

// used after highland batch step
const toBatchUow = batch => ({
  batch
}); // use with flatMap


exports.toBatchUow = toBatchUow;

const unBatchUow = uow => {
  const {
    batch,
    ...outerUowMinusBatch
  } = uow;
  return batch.map(innerUowFromBatch => ({ ...innerUowFromBatch,
    ...outerUowMinusBatch
  }));
};

exports.unBatchUow = unBatchUow;

const group = rule => {
  /* istanbul ignore if */
  if (!rule.group) return s => s;
  return s => s.group(uow => uow.event.partitionKey).flatMap(toGroupUows);
}; // use with flatMap after highland group step


exports.group = group;

const toGroupUows = groups => (0, _highland.default)(Object.keys(groups).map(key => ({
  batch: groups[key]
})));

exports.toGroupUows = toGroupUows;

const batchWithSize = opt => {
  let batched = [];
  let sizes = [];
  return (err, x, push, next) => {
    /* istanbul ignore if */
    if (err) {
      push(err);
      next();
    } else if (x === _highland.default.nil) {
      if (batched.length > 0) {
        logMetrics(batched, sizes, opt);
        push(null, batched);
      }

      push(null, _highland.default.nil);
    } else {
      const size = Buffer.byteLength(JSON.stringify(x[opt.requestEntryField]));
      const totalSize = sizes.reduce((a, c) => a + c, size);

      if (totalSize <= opt.maxRequestSize && batched.length + 1 <= opt.batchSize) {
        batched.push(x);
        sizes.push(size);
      } else {
        logMetrics(batched, sizes, opt);
        push(null, batched);
        batched = [x];
        sizes = [size];
      }

      next();
    }
  };
};

exports.batchWithSize = batchWithSize;

const logMetrics = (batch, sizes, opt) => {
  if (opt.metricsEnabled) {
    opt.debug('%j', {
      metrics: {
        [opt.requestField]: {
          count: sizes.length,
          ...sizes.reduce((a, size, i) => ({ ...a,
            average: (a.sum + size) / (i + 1),
            min: a.min < size ? a.min : size,
            max: a.max > size ? a.max : size,
            sum: a.sum + size,
            types: { ...a.types,
              [batch[i][opt.requestEntryField].DetailType]: [...(a.types[batch[i][opt.requestEntryField].DetailType] || []), size]
            }
          }), {
            average: 0,
            min: undefined,
            max: undefined,
            sum: 0,
            types: {}
          })
        }
      }
    });
  }
};