"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toRecord = exports.publishToKinesis = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _kinesis = _interopRequireDefault(require("../connectors/kinesis"));

var _batch = require("./batch");

var _faults = require("./faults");

var _print = require("./print");

var _tags = require("./tags");

const publishToKinesis = ({
  debug = (0, _print.debug)('kinesis'),
  streamName = process.env.STREAM_NAME,
  eventField = 'event',
  batchSize = Number(process.env.PUBLISH_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 25,
  parallel = Number(process.env.PUBLISH_PARALLEL) || Number(process.env.PARALLEL) || 8,
  handleErrors = true
} = {}) => {
  const connector = new _kinesis.default({
    debug,
    streamName
  });

  const toInputParams = batchUow => ({ ...batchUow,
    inputParams: {
      Records: batchUow.batch.filter(uow => uow[eventField]).map(uow => toRecord(uow[eventField]))
    }
  });

  const putRecords = batchUow => {
    if (!batchUow.inputParams.Records.length) {
      return (0, _highland.default)(Promise.resolve(batchUow));
    }

    const p = connector.putRecords(batchUow.inputParams).then(publishResponse => ({ ...batchUow,
      publishResponse
    })).catch((0, _faults.rejectWithFault)(batchUow, !handleErrors));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map((0, _tags.adornStandardTags)(eventField)).batch(batchSize).map(_batch.toBatchUow).map(toInputParams).map(putRecords).parallel(parallel).flatMap(_batch.unBatchUow); // for cleaner logging and testing
};

exports.publishToKinesis = publishToKinesis;

const toRecord = e => ({
  Data: Buffer.from(JSON.stringify(e)),
  PartitionKey: e.partitionKey
});

exports.toRecord = toRecord;